      subroutine rotate(fft_2d, nfc_2d_row, nfc_2d_col, fc_2d,
     +     omega, annul)
      
C     Rotate anticlockwise 2D fft plane fft_2d with omega angle and save
C     to fc_2d. omega is in degree, and only rotate up to certain
C     annulus.

      implicit none
      include '../include/allprogs.inc'

      integer, intent(in) :: nfc_2d_row, nfc_2d_col
      real, intent(in) :: omega, annul
      double precision, intent(in)  :: 
     $     fft_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      double precision, intent(out) :: 
     $     fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      
      real sinom, cosom, ru, rv, radius, tmpang, fu, fv,
     $     du_minus, du_plus, dv_minus, dv_plus, del_u, del_v
      integer  u, v, iu_minus, iu_plus, iv_minus, iv_plus, 
     $     iu_nest, iv_nest
      double precision fc_nw, fc_ne, fc_sw, fc_se
      
      real*8 dg_to_rd, rd_to_dg
      dg_to_rd = acos(-1.0d0)/180.0d0
      rd_to_dg = 1.0d0/dg_to_rd

      fc_2d(0,0) = fft_2d(0,0)

      do v = -nfc_2d_col, nfc_2d_col 
         fv = real(v)
         do u = 0, nfc_2d_row
            fu = real(u)
            radius = sqrt(fu*fu + fv*fv)
            if (radius <= 0.0  .or. radius > annul) then
               fc_2d(u, v) =  fft_2d(u, v)
            else
               if (u > 0) then
                  tmpang = rd_to_dg * atan2(fv, fu)
               else 
                  if (v >= 0)  then 
                     tmpang = 90.0
                  else
                     tmpang = -90.0
                  endif
               endif

C     Calculate old angle
               sinom = sin((tmpang - omega)*dg_to_rd)
               cosom = cos((tmpang - omega)*dg_to_rd)

C     Calculate old u coordinate
               ru       =  radius * cosom
               iu_nest  = nint(ru)
               del_u         = ru - real(iu_nest)
               if(del_u >= 0.0) then
                  du_minus    = del_u
                  iu_minus    = iu_nest
               else
                  du_minus    = 1.0 + del_u
                  iu_minus    = iu_nest - 1
               endif
               du_plus       = 1.0 - du_minus
               iu_plus       = 1   + iu_minus

C     Calculate old v coordinate
               rv       = radius * sinom
               iv_nest  = nint(rv)
               del_v         = rv - real(iv_nest)
               if(del_u >= 0.0) then
                  dv_minus    = del_v
                  iv_minus    = iv_nest
               else
                  dv_minus    = 1.0 + del_v
                  iv_minus    = iv_nest - 1
               endif
               dv_plus       = 1.0 - dv_minus
               iv_plus       = 1 + iv_minus

               if (iu_minus < -nfc_2d_row)  iu_minus = iu_plus
               if (iv_minus < -nfc_2d_col)  iv_minus = iv_plus
               if (iu_plus  > nfc_2d_row)   iu_plus  = iu_minus
               if (iv_plus  > nfc_2d_col)   iv_plus  = iv_minus
               
               if(iu_minus < 0) then
                  fc_nw =  fft_2d(-iu_minus, -iv_minus) 
                  fc_ne =  fft_2d(-iu_minus, -iv_plus) 
               else
                  fc_nw = fft_2d(iu_minus, iv_minus)
                  fc_ne = fft_2d(iu_minus, iv_plus)
               endif

               if(iu_plus < 0) then
                  fc_sw =  fft_2d(-iu_plus, -iv_minus)
                  fc_se =  fft_2d(-iu_plus, -iv_plus) 
               else
                  fc_sw = fft_2d(iu_plus, iv_minus)
                  fc_se = fft_2d(iu_plus, iv_plus)
               endif

               fc_2d(u,v)= fc_se*du_minus*dv_minus + 
     +              fc_sw*du_minus*dv_plus +
     +              fc_ne*du_plus*dv_minus + 
     +              fc_nw*du_plus*dv_plus
            endif 

         enddo
      enddo   

      return
      end subroutine rotate
