C  ****************************************************************************
C  *                                                                          *
C  *                     *****     EMMAPZOOM.FOR     *****                    *
C  *                                                                          *
C  *    ZOOMS PORTION OR WHOLE 3D MAP UP OR DOWN.                             *
C  *                                                                          *
C  *    THIS PROGRAM IS A HYBRID OF EMMAP3DT AND EMSF3DBT.                    *
C  *    OUTPUT IS A 3D MAP FILE IN PIF FORMAT.                                *
C  *                                                                          *
C  *    UNITS: 1 = 3D MAP INPUT DATA FILE                                     *
C  *           2 = 3D I*2 MAP OUTPUT FILE                                     *
C  *                                                                          *
C  *    OCTOBER 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
C  *                                                                          *
C  *    MODIFIED  1-NOV-1998 BY TSB (Merge old EMMAP3DT & EMSF3DBT).          *
C  *    LAST MODIFIED 16-NOV-2001 BY TSB (Linux version).                     *
C  ****************************************************************************
C
      IMPLICIT NONE

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, LMAX, D(5), NSF
      INTEGER*4 NUM_UNIQUE, KMIN, NCD, NEF, T_MATRIX(2,3), TABLE_SIZE
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
      REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
      REAL*4    TRIG_TABLE(1001), DCELL(9), MAP_MIN, MAP_MAX
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ
      character(len=1) MAP_TITLE(80)
      CHARACTER MAP_FILENAME*256
      integer alloc_stat

      INTEGER*2, allocatable :: HKL(:)
      real*4, allocatable :: mapin(:), array(:)
      REAL*4, allocatable :: ARRAY2(:), TABLE1(:), TABLE2(:)
      REAL*4, allocatable :: MAPOUT(:), AP(:), BP(:)
      REAL*4, allocatable :: SINHX(:), COSHX(:), SINKY(:), COSKY(:)

C  ----------------------------------------------------------------------------
C
      PRINT *, 'EMMAPZOOM: Zoom 3D MAP larger or smaller'
Ca      PRINT *, '           20-Nov-2001 [TSB]'
      call banner('EMMAPZOOM\0')
      PRINT *
C
      CALL INFO_EMMA (NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF, A11, A21,
     $     A22,A31, A32, A33, A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
     $     ,A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR,
     $     VOL_STAR,DSQMIN, DSQMAX, SCALE_FAC, MAP_FILENAME, MAP_TITLE,
     $     TRIG_TABLE,DCELL, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX,
     $     ZMAX)

      allocate(mapin(ncol*nrow*nsec), array((ncol+4)*nrow*nsec),
     $     stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAPIN and ARRAY in EMMAPZOOM'
         call tsbend
      endif

C  ***********************************
C  *    READ IN THE ENTIRE 3D MAP    *
C  ***********************************
      CALL PIF_READ_TMAPI2 (MAPHEAD, MAPIN, NCOL, NROW, NSEC, 1, 1, 1)
C  *****************************************************************
C  *    CALCULATE FOURIER TRANSFORMS ON Z AND X OF NSEC SECTIONS.  *
C  *****************************************************************
      CALL MAP3DT_PASS1 (MAPIN, ARRAY, NCOL, NROW, NSEC, D, HMAX)
      deallocate(mapin)
      CALL PIF_CLOSE (MAPHEAD%LUNUM) ! CLOSE AND SAVE INPUT 3D MAP FILE

      allocate(ARRAY2(2*NSEC*NROW*(HMAX+1)), HKL(3*NSF),
     $     AP(NSF), BP(NSF), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate ARRAY2, HKL, AP, BP'
         call tsbend
      endif

      CALL MAP3DT_STORE (ARRAY, NCOL, NROW, NSEC, HMAX, KMAX, ARRAY2)
C  ************************************************************************
C  *   READ INTERMEDIATE RESULTS AND CALCULATE FOURIER TRANSFORMS ON Y.   *
C  ************************************************************************
      CALL MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)

      CALL SFLOAD (ARRAY, HKL, AP, BP, NSEC, HMAX, KMAX, LMAX, NSF, 
     1     A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC,
     2     NUM_UNIQUE)

      CALL RIP (NCOL, NROW, NSEC, NCD, NEF, HKL, NUM_UNIQUE, AP, BP,
     $     DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DCELL, 
     $     HMAX, KMIN,KMAX)

      TABLE_SIZE = NSEC*NCD

      allocate(MAPOUT(NCOL*NROW*NSEC), TABLE2(NEF), 
     $     SINHX(NCOL*(HMAX+1)), COSHX(NCOL*(HMAX+1)),
     $     SINKY(NROW*(KMAX-KMIN+1)), COSKY(NROW*(KMAX-KMIN+1)),
     $     TABLE1(TABLE_SIZE), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate MAPOUT etc.'
         call tsbend
      endif

C  ***********************************************
C  *    CALCULATE FOURIER, SECTION BY SECTION    *
C  ***********************************************
      CALL FOURIER (MAPOUT, TRIG_TABLE, TABLE_SIZE, NCD, NEF, TABLE1, 
     1 TABLE2, NCOL, NROW, NSEC, AP, BP, HKL, SINHX, COSHX, SINKY, 
     2 COSKY, MAP_MIN, MAP_MAX, NUM_UNIQUE, DX, DY, DZ, XMIN, 
     3 YMIN, ZMIN, HMAX, KMIN, KMAX)

      deallocate(ap, bp)

      deallocate(SINHX, COSHX, SINKY, COSKY, TABLE1, TABLE2)

      CALL MAP_OUT (MAPOUT, NCOL, NROW, NSEC, A_CELL, B_CELL, C_CELL, 
     1     ALP, BET, GAM, T_MATRIX, MAP_MIN, MAP_MAX, MAP_FILENAME,
     2     MAP_TITLE)

      deallocate(mapout, array, array2, hkl)

      CALL TSBEND
      END

C  ****************************************************************
C  *                                                              *
C  *              *****     SUBROUTINE INFO     *****             *
C  *                                                              *
C  *    ROUTINE TO READ IN CONTROL PARAMETERS                     *
C  ****************************************************************
C
      SUBROUTINE INFO_EMMA(NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF, A11,
     $     A21, A22, A31, A32, A33, A_CELL, B_CELL, C_CELL, ALP, BET,
     $     GAM,VOL, A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
     $     ,VOL_STAR, DSQMIN, DSQMAX, SCALE_FAC, MAP_FILENAME, MAP_TITLE
     $     ,TRIG_TABLE, DCELL, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX
     $     ,ZMAX)
C
      IMPLICIT NONE
C
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*2 IH, IK, IL
      INTEGER*4 I, M, N, NSF
      INTEGER*4 HMAX, KMAX, LMAX, NCOL, NROW, NSEC, NC2, NR2, NS2
      REAL*4    V, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX
      REAL*4    AA(3,3), BB(3,3)
      REAL*4    DCELL(9), A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
      REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR, TRIG_TABLE(1001)
      REAL*4    ALP_STAR, BET_STAR, GAM_STAR, RCELL(9)
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    PI, RADIAN, DEGREE, RESLO, RESHI, PIXSIZ_OLD, PIXSIZ_NEW
      REAL*4    COSALP, SINALP, COSBET, SINBET, COSGAM, SINGAM
      REAL*4    COSALP_STAR, COSBET_STAR, COSGAM_STAR
      REAL*4    SINALP_STAR, SINBET_STAR, SINGAM_STAR
      REAL*4    AST, BST, CST, TMP, XSQ, YSQ, ZSQ, DSTAR, MODF, PHI
      REAL*8    DPI, DRADIAN, DDEGREE, ARG, DEL_ARG
      real*8    deg_to_rad
      CHARACTER MAP_FILENAME*256
      character(len=1) READWRITE(4), MAP_TITLE(80)
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      NSF = 0                  ! SET COUNTER FOR # STRUCTURES FACTORS CALCULATED
        CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ***********************
C  *    OPEN MAP FILE    *
C  ***********************
      WRITE(*,'(A)',advance='no') 'Input 3D map filename: '
      READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
      MAPHEAD%LUNUM = 1
      READWRITE(1)  = 'r'
      CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, READWRITE) 
      CALL PIF_READ_GH (MAPHEAD)
      IF(MAPHEAD%MODE.EQ.20) THEN
         MAPHEAD%IMGNUM = 1                              ! LOAD UP "IMG" #1
         CALL PIF_READ_DH (MAPHEAD)
         NCOL   = MAPHEAD%NUMCOLS
         NROW   = MAPHEAD%NUMROWS
         NSEC   = MAPHEAD%NUMSECS
         A_CELL = MAPHEAD%A_CELL
         B_CELL = MAPHEAD%B_CELL
         C_CELL = MAPHEAD%C_CELL
       ELSE
         PRINT *, 'Invalid PIF mode for 3D MAP'
         CALL TSBEND
      ENDIF
C  *************************************
C  *    ENTER NAME OF FINAL MAP FILE   *
C  *************************************
      WRITE(*,'(A)',advance='no') 'Output 3D map filename: '
      READ (*,'(A)') MAP_FILENAME
      WRITE(*,'(A)',advance='no') 'Title: '
      READ (*,'(80A1)') MAP_TITLE
C  ********************************************************************
C  *    CHECK INPUT FOR NCOL = EVEN RESTRICTION                       *
C  *    AND ALSO THAT NO PRIMES > 19 ARE FACTORS OF THE DIMENSIONS    *
C  ********************************************************************
      CALL CHK_PRIME (NCOL, NC2, .TRUE.)
      CALL CHK_PRIME (NROW, NR2, .FALSE.)
      CALL CHK_PRIME (NSEC, NS2, .FALSE.)
      WRITE(6,'(A,3I5)') ' 3D MAP: input dimensions =', NCOL, NROW, NSEC
      WRITE(6,'(A,3I5)') ' 3D MAP:   new dimensions =', NC2, NR2, NS2
      NCOL = NC2
      NROW = NR2
      NSEC = NS2
      PRINT *, ' Enter pixsiz(new & old) and res limits as pixels, ',
     1 'Angstroms or nms'
      WRITE(*,'(2A)',advance='no') 'PIXSIZ_OLD,PIXSIZ_NEW,RES_HI',
     1 ' <CR>=1.0,1.0,2.00001: '
      READ (*,*) PIXSIZ_OLD, PIXSIZ_NEW, RESHI
      IF(PIXSIZ_OLD.LE.0.0) PIXSIZ_OLD = 1.0
      IF(PIXSIZ_NEW.LE.0.0) PIXSIZ_NEW = 1.0
      IF(RESHI.LE.0.0)           RESHI = 2.00001*PIXSIZ_OLD
      RESLO     = 0.0
      SCALE_FAC = 1.0
      DSQMIN    = 0.0
      DSQMAX    = 1.0/(RESHI**2)
      WRITE(*,'(A)',advance='no') 'XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX: '
      READ (*,*) XMIN, XMAX, YMIN, YMAX, ZMIN, ZMAX
      IF(XMAX.EQ.0.0) XMAX = 1.0
      IF(YMAX.EQ.0.0) YMAX = 1.0
      IF(ZMAX.EQ.0.0) ZMAX = 1.0
      DX = PIXSIZ_NEW
      IF(DX.LE.0.0) DX = 1.0
      DY = DX
      DZ = DX
C
C      HARDWIRE THESE FOR NOW
C
      A_CELL = NC2*PIXSIZ_OLD                  !!! FIX: these are WRONG !!!!!
      B_CELL = NR2*PIXSIZ_OLD
      C_CELL = NS2*PIXSIZ_OLD
      ALP    = 90.0
      BET    = 90.0
      GAM    = 90.0
      CALL REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
     1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
   10      WRITE(*,1000) A_CELL,B_CELL,C_CELL,ALP,BET,GAM,RESHI,RESLO
      WRITE(*,'(T12,A,F10.2)') ' Fourier scaled by', SCALE_FAC
C  *************************************************************
C  *    CALCULATE RECIPROCAL LATTICE PARAMETERS                *
C  *    (SEE INTERNATIONAL TABLES FOR CRYSTALLOGRAPHY, P.13)   *
C  *************************************************************
      COSALP      = COS(ALP*deg_to_rad)
      SINALP      = SIN(ALP*deg_to_rad)
      COSBET      = COS(BET*deg_to_rad)
      SINBET      = SIN(BET*deg_to_rad)
      COSGAM      = COS(GAM*deg_to_rad)
      SINGAM      = SIN(GAM*deg_to_rad)
      COSALP_STAR = (COSBET*COSGAM-COSALP)/(SINBET*SINGAM)
      COSBET_STAR = (COSGAM*COSALP-COSBET)/(SINGAM*SINALP)
      COSGAM_STAR = (COSALP*COSBET-COSGAM)/(SINALP*SINBET)
      SINALP_STAR = SQRT(1.0-COSALP_STAR**2)
      SINBET_STAR = SQRT(1.0-COSBET_STAR**2)
      SINGAM_STAR = SQRT(1.0-COSGAM_STAR**2)
      AST         = 1.0/(A_CELL*SINBET*SINGAM_STAR)
      BST         = 1.0/(B_CELL*SINGAM*SINALP_STAR)
      CST         = 1.0/(C_CELL*SINALP*SINBET_STAR)
C  ***************************************
C  *    CALCULATE MATRIX FOR D* TESTS    *
C  ***************************************
      A11 =  BST*SINGAM_STAR*SINALP
      A21 = -BST*SINGAM_STAR*COSALP
      A22 =  CST*SINBET_STAR
      A31 =  BST*COSGAM_STAR
      A32 =  CST*COSBET_STAR
      A33 =  AST
C  ******************************************
C  *     CALCULATE LIMITS ON H, K, AND L    *
C  ******************************************
      HMAX = INT(SINALP_STAR/(AST*RESHI*SQRT(SINALP_STAR**2 + 
     1      (SINBET_STAR*COSGAM)**2 + (SINGAM_STAR*COSBET)**2 + 
     2      2.0*COSALP_STAR*(2.0*COSBET_STAR*COSGAM_STAR +
     3      SINBET_STAR*COSBET*SINGAM_STAR*COSGAM) - 
     4      2.0*(COSBET_STAR**2+COSGAM_STAR**2))))
      KMAX = INT(SINBET_STAR/(BST*RESHI*SQRT(SINBET_STAR**2 + 
     1      (SINGAM_STAR*COSALP)**2 + (SINALP_STAR*COSGAM)**2 +
     2      2.0*COSBET_STAR*(2.0*COSGAM_STAR*COSALP_STAR +
     3      SINGAM_STAR*COSGAM*SINALP_STAR*COSALP) - 
     4      2.0*(COSALP_STAR**2+COSGAM_STAR**2))))
      LMAX = INT(SINGAM_STAR/(CST*RESHI*SQRT(SINGAM_STAR**2 +
     1      (SINBET_STAR*COSALP)**2 + (SINALP_STAR*COSBET)**2 +
     2      2.0*COSGAM_STAR*(2.0*COSBET_STAR*COSALP_STAR +
     3      SINBET_STAR*COSBET*SINALP_STAR*COSALP) -
     4      2.0*(COSBET_STAR**2+COSALP_STAR**2))))
C
      WRITE(*,1010) NC2, NR2, NS2, HMAX, KMAX, LMAX
C
      SCALE_FAC = SCALE_FAC/REAL(NC2*NR2*NS2)
C  ****************************************************
C  *    CHECK INPUT FOR POSSIBLE PROGRAM OVERFLOWS    *
C  ****************************************************
      IF(NC2.LT.2*HMAX+1) THEN
        WRITE(*,1020) HMAX, NC2
        CALL TSBEND
      ENDIF
      IF(NR2.LT.2*KMAX+1) THEN
        WRITE(*,1030) KMAX, NR2
        CALL TSBEND
      ENDIF
      IF(NS2.LT.2*LMAX+1) THEN
        WRITE(*,1040) LMAX, NS2
        CALL TSBEND
      ENDIF
C  ***************************************************************
C  *    DETERMINE MAXIMUM SIZE OF NSF WITHIN RESOLUTION LIMITS   *
C  ***************************************************************
        DO IK = -KMAX,KMAX
          XSQ = (REAL(IK)*A11)**2
          DO IL = -LMAX,LMAX
            YSQ = (REAL(IK)*A21 + REAL(IL)*A22)**2 
            DO IH = 1,HMAX+1
              ZSQ = (REAL(IH-1)*A33 + REAL(IK)*A31 + REAL(IL)*A32)**2
              DSTAR = XSQ + YSQ + ZSQ
              IF(DSTAR.LE.DSQMAX.AND.DSTAR.GE.DSQMIN) NSF = NSF + 1
            ENDDO                               ! DO IH = 1,HMAX+1
          ENDDO                               ! DO IL = 1,(2*LMAX+1)
        ENDDO                                     ! DO IK = 1,(2*KMAX+1)
C
      WRITE(*,'(12X,A,I9)') 'Max # of structure factors computed =', NSF
C
      CALL PIRADDEG_D (DPI, DRADIAN, DDEGREE)
      DEL_ARG = 2.0*DPI/1000.0
      ARG     = 0.D0
C
      DO I = 1,1001
        TRIG_TABLE(I) = COS(SNGL(ARG))
        ARG           = ARG + DEL_ARG
      ENDDO
C
      DCELL(1) = A_CELL
      DCELL(2) = B_CELL
      DCELL(3) = C_CELL
      DCELL(4) = ALP
      DCELL(5) = BET
      DCELL(6) = GAM
      IF(DCELL(4).GE.1.0) THEN
        DO I = 4,6
          DCELL(I) = COS(DCELL(I)/DDEGREE)
        ENDDO
      ENDIF
      CALL SF3DBT_STOAA (DCELL, AA)
      CALL SF3DBT_MI3 (AA, BB, V)
      V = SQRT(V)
      RCELL(1) = SQRT(BB(1,1))
      RCELL(2) = SQRT(BB(2,2))      
      RCELL(3) = SQRT(BB(3,3))
      RCELL(4) = BB(2,3)/(RCELL(2)*RCELL(3))
      RCELL(5) = BB(1,3)/(RCELL(1)*RCELL(3))
      RCELL(6) = BB(1,2)/(RCELL(1)*RCELL(2))
      DO I = 4,6
        DCELL(I+3) = ATAN2(SQRT(1.-DCELL(I)*DCELL(I)),DCELL(I))*DDEGREE
        RCELL(I+3) = ATAN2(SQRT(1.-RCELL(I)*RCELL(I)),RCELL(I))*DDEGREE
      ENDDO
      IF(DCELL(1).LT.1.0) THEN
        V = 1.0/V
        DO I = 1,9
          CALL SF3DBT_ROT (DCELL(I), RCELL(I))
          CALL SF3DBT_ROT ( AA(I,1),  BB(I,1))
        ENDDO
      ENDIF
C
      PRINT *, 'V,VOL,VOL_STAR', V,VOL,VOL_STAR
      WRITE(*,1050) DCELL
      WRITE(*,1060) RCELL
      DX = DX/DCELL(1)
      DY = DY/DCELL(2)
      DZ = DZ/DCELL(3)
C
      CALL CHECKUP (XMIN, XMAX, DX)
      CALL CHECKUP (YMIN, YMAX, DY)
      CALL CHECKUP (ZMIN, ZMAX, DZ)
      RETURN
 999  CALL TSBEND
C     
 1000 FORMAT(/,20X,'STRUCTURE FACTOR CALCULATION',/,20X,28('-'),/,12X,
     1     '    A =',F7.2,'     B =',F7.2,'      C =',F7.2,/,12X,
     2     'ALPHA =',F7.2,'  BETA =',F7.2,'  GAMMA =',F7.2,/,12X,
     3     'Resolution limits are',F7.2,' to ',F7.2)
 1010 FORMAT(12X,'Input cell divisions are',3I5,' pixels',/,12X,
     1     'Limits on H, K and L are',3I5)
 1020 FORMAT(' HMAX out of range.  HMAX equals',I5,' NCOL equals',I5)
 1030 FORMAT(' KMAX out of range.  KMAX equals',I5,' NROW equals',I5)
 1040 FORMAT(' LMAX out of range.  LMAX equals',I5,' NSEC equals',I5)
 1050 FORMAT(/,5X,'A',7X,'B',7X,'C',5X,'COS ALP',2X,'COS BET',2X,
     1     'COS GAM',3X,'ALP',4X,'BET',4X,'GAM',/' ',3F8.2,3F9.5,3F7.2)
 1060 FORMAT(/,5X,'A*',6X,'B*',6X,'C*',4X,'COS ALP*',1X,'COS BET*',1X,
     1     'COS GAM*',3X,'ALP*',3X,'BET*',3X,'GAM*',/  ' ',
     2     3F8.5,3F9.5,1X,3F7.2)
      END

C  **************************************************************************
C  *                                                                        *
C  *                    *****     SUBROUTINE RIP     *****                  *
C  *                                                                        *
C  *    ROUTINE CALLED BY "EMSF3DBT" PROGRAM                                *
C  *                                                                        *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Eliminate need for EMSF3DBT.TMP   *
C  *     scratch file for storing intermediate SF data).                    *
C  **************************************************************************
C
      SUBROUTINE RIP (NCOL, NROW, NSEC, NCD, NEF, HKL, NUM_UNIQUE, 
     1 AP, BP, DX, DY, DZ, XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DCELL, 
     2 HMAX, KMIN, KMAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NUM_UNIQUE, I, M, N, JJ
      INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, KMIN, NN, NCD, NEF
      INTEGER*2 HKL(3,NUM_UNIQUE), J, K, L
      REAL*4    XMIN, YMIN, ZMIN, XMAX, YMAX, ZMAX, DX, DY, DZ, DCELL(9)
      REAL*4    APART, BPART
      REAL*4    AP(NUM_UNIQUE), BP(NUM_UNIQUE), FJX, FKX, FLX
      real*8    deg_to_rad

      character(len=1) xl, yl, zl
      DATA      XL/'X'/, YL/'Y'/, ZL/'Z'/
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0

      HMAX   = 0
      KMAX   = 0
      KMIN   = 0
C
      FJX = DX*DCELL(1)
      FKX = DY*DCELL(2)
      FLX = DZ*DCELL(3)
      WRITE(*,1000) XL, XMIN, XMAX, DX, FJX, YL, YMIN, YMAX, DY, FKX,
     1  ZL, ZMIN, ZMAX, DZ, FLX
C  ********************************
C  *    SET NEW MAP DIMENSIONS    *
C  ********************************
      NCOL = (XMAX-XMIN)/DX+1.5
      NROW = (YMAX-YMIN)/DY+1.5
      NSEC = (ZMAX-ZMIN)/DZ+1.5
      WRITE(*,'(3(A,I6))') ' MAP dimensions:', NCOL, ' by', NROW,
     1 ' by', NSEC
C
      DO 800 NN = 1,NUM_UNIQUE
        J     = HKL(1,NN)
        K     = HKL(2,NN)
        L     = HKL(3,NN)
        APART = AP(NN)*COS(deg_to_rad*BP(NN))
        BPART = AP(NN)*SIN(deg_to_rad*BP(NN))
ccc        APART = AP(NN)
ccc        BPART = BP(NN)
  600        CONTINUE
  640        IF(HKL(1,NN)) 680,650,710
  650        IF(HKL(2,NN)) 690,660,710
  660        IF(HKL(3,NN)) 700,670,710
  670        GO TO 710
  680        J = -J
  690        K = -K
  700        L = -L
        BPART = -BPART
  710        IF(APART.NE.0.0 .OR. BPART.NE.0.0) THEN
          IF(J.GT.HMAX) HMAX = J
          IF(K.GT.KMAX) KMAX = K
          IF(K.LT.KMIN) KMIN = K
        ENDIF
        HKL(1,NN) = J
        HKL(2,NN) = K
        HKL(3,NN) = L
        AP(NN)    = APART
        BP(NN)    = BPART
  800      ENDDO
C
      NCD = (HMAX+1)*(KMAX-KMIN+1)*2
      NEF = (HMAX+1)*NROW*2
      WRITE(*,1020) NCD, NEF
      RETURN
C
 1000      FORMAT(' ',A2,'Limits: ',F8.5,' to',F8.5,' in steps of',
     1  F8.5,' (',F6.3,' nms)')
 1010      FORMAT(I8,' independent reflections')
 1020      FORMAT(' The CD table requires',I6,' memory locations'/,
     1  ' The EF table requires',I6,' memory locations'/)
      END

C  **************************************************************************
C  *                                                                        *
C  *                     *****     EMSF3DBT.SUBS     *****                  *
C  *                                                                        *
C  *    CONTAINS THE FOLLOWING SUBROUTINES:                                 *
C  *                                                                        *
C  *                 FOURIER                                                *
C  *                 RIP                                                    *
C  *                 CHECKUP                                                *
C  *                 MAP_OUT                                                *
C  **************************************************************************
C
C
C

C  *******************************************************************
C  *                                                                 *
C  *              *****     SUBROUTINE FOURIER     *****             *
C  *                                                                 *
C  *    FOURIER SUMMATION ROUTINE                                    *
C  *                                                                 *
C  *    THIS IS A MAJOR REVISION OF THE SUBROUTINE "SIG100"          *
C  *     OF THE PROGRAM FOUR3D.                                      *
C  *                                                                 *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Store MAP in 3D array and  *
C  *     eliminate need for EMSF3DBT.SCR scratch file).              *
C  *******************************************************************
C
      SUBROUTINE FOURIER (MAP, TRIG_TABLE, TABLE_SIZE, NCD, NEF, TABLE1,
     1 TABLE2, NCOL, NROW, NSEC, AP, BP, HKL, SINHX, COSHX, SINKY, 
     2 COSKY, MAP_MIN, MAP_MAX, NSF, DX, DY, DZ, XMIN, YMIN, ZMIN, 
     3 HMAX, KMIN, KMAX)
C
      IMPLICIT NONE

      INTEGER*4 TABLE_SIZE, NCOL, NROW, NSEC, NEF, NSF
      INTEGER*4 H, I, J, K, L, M, N, CE, CF, LOCEI, CC, CD
      INTEGER*4 HMAX, KMAX, KMIN, NH, NK, NCD, ISEC
      integer*4 SECTION_THIS_LOOP
      INTEGER*2 HKL(3,NSF), NSECS_THIS_LOOP
      REAL*4    TABLE1(TABLE_SIZE), TABLE2(NEF), TRIG_TABLE(1001)
      REAL*4    SINKY(NROW*(KMAX-KMIN+1)), COSKY(NROW*(KMAX-KMIN+1))
      REAL*4    SINHX(NCOL*(HMAX+1)), COSHX(NCOL*(HMAX+1))
      REAL*4    MAP_MIN, MAP_MAX, XMIN, YMIN, ZMIN, DX, DY, DZ
      REAL*4    AP(NSF), BP(NSF), MAP(NCOL,NROW,NSEC)
      REAL*4    ARG, SINARG, COSARG, X, Y, Z, DEL_ARG
C  ----------------------------------------------------------------------------
C
      MAP_MIN         = HUGE(MAP_MIN)
      MAP_MAX         = -HUGE(MAP_MAX)
      ISEC            = 0
      NH              = HMAX + 1            ! # H INDICES
      NK              = KMAX - KMIN + 1      ! # OF K INDICES
      NSECS_THIS_LOOP = TABLE_SIZE/NCD      ! # SECTIONS HANDLED EACH LOOP
      Z               = ZMIN
      PRINT *, ' '
      WRITE(*,'(A)') '   Set up SINHX,COSHX tables'
C  ***********************************
C  *    SET UP SINHX,COSHX TABLES    *
C  ***********************************
      I       = 1
      DEL_ARG = 0.0
      X       = 0.0
      DO N = 1,NH
        ARG  = X
        DO M = 1,NCOL
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          SINHX(I) = SINARG
          COSHX(I) = COSARG
          I        = I + 1
          ARG      = ARG + DEL_ARG
        ENDDO
        DEL_ARG = DEL_ARG + DX
        X       = X + XMIN
      ENDDO
      WRITE(*,'(A)') '   Set up SINHX,COSHX tables'
C  ***********************************
C  *    SET UP SINKY,COSKY TABLES    *
C  ***********************************
      I       = 1
      Y       = REAL(KMIN)*YMIN
      DEL_ARG = REAL(KMIN)*DY
      DO K = 1,NK
        ARG  = Y
        DO N = 1,NROW
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          SINKY(I) = SINARG
          COSKY(I) = COSARG
          I        = I + 1
          ARG      = ARG + DEL_ARG
        ENDDO
        Y       = Y + YMIN
        DEL_ARG = DEL_ARG + DY
      ENDDO
C
 100  IF(NSECS_THIS_LOOP.GT.NSEC-ISEC) NSECS_THIS_LOOP = NSEC - ISEC
C  **************************
C  *    INITIALIZE TABLE    *
C  **************************
      WRITE(*,'(A)') '   Set up TABLE1            '
      DO I = 1,TABLE_SIZE
        TABLE1(I) = 0.0
      ENDDO
C  **********************
C  *    SET UP TABLE    *
C  **********************
      DO I = 1,NSF
        CC      = HKL(1,I)*NK + (HKL(2,I) - KMIN) + 1
        CD      = CC + NCD/2
        ARG     = HKL(3,I)*Z
        DEL_ARG = HKL(3,I)*DZ
        DO J = 1,NSECS_THIS_LOOP
          CALL SF3DBT_GETARGS (ARG, SINARG, COSARG, TRIG_TABLE)
          TABLE1(CC) = TABLE1(CC) + AP(I)*COSARG + BP(I)*SINARG
          TABLE1(CD) = TABLE1(CD) - AP(I)*SINARG + BP(I)*COSARG
          CC         = CC + NCD
          CD         = CC + NCD/2
          ARG        = ARG + DEL_ARG
        ENDDO
      ENDDO
C
  300      CC = 1
      CD = CC + NCD/2
      DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
        ISEC = ISEC + 1
        if(section_this_loop.lt.nsecs_this_loop) then
          if(mod(section_this_loop,10).eq.0) then
            WRITE(*,'(A,I4)') 'Fourier section', ISEC
          endif
        endif
        DO J = 1,NEF
          TABLE2(J) = 0.0
        ENDDO
        LOCEI = 1
        DO J = 1,NH
          I = 1
          DO K = 1,NK
            CE = LOCEI
            CF = CE + NEF/2
            DO N = 1,NROW
              TABLE2(CE) = TABLE2(CE) + TABLE1(CC)*COSKY(I)
     1                               + TABLE1(CD)*SINKY(I)
              TABLE2(CF) = TABLE2(CF) - TABLE1(CC)*SINKY(I)
     1                               + TABLE1(CD)*COSKY(I)
              CE = CE + HMAX + 1
              CF = CE + NEF/2
              I  = I + 1
            ENDDO
            CC    = CC + 1
            CD    = CC + NCD/2
          ENDDO
          LOCEI = LOCEI + 1
        ENDDO
C
        CE = 1
        CF = NEF/2 + 1
        DO J = 1,NROW
          DO K = 1,NCOL                              ! ZERO ROW OF MAP
            MAP(K,J,ISEC) = 0.0
          ENDDO
          I = 1
          DO N = 1,NH
            DO M = 1,NCOL
              MAP(M,J,ISEC) = MAP(M,J,ISEC) + TABLE2(CE)*COSHX(I)
     1                                     + TABLE2(CF)*SINHX(I)
              I      = I + 1
            ENDDO
            CE = CE + 1
            CF = CE + NEF/2
          ENDDO
          DO M = 1,NCOL
            MAP_MIN = AMIN1(MAP_MIN,MAP(M,J,ISEC))
            MAP_MAX = AMAX1(MAP_MAX,MAP(M,J,ISEC))
          ENDDO
        ENDDO
        CC   = CC + NCD/2
        CD   = CC + NCD/2
        Z    = Z + DZ
      ENDDO                       ! DO SECTION_THIS_LOOP = 1,NSECS_THIS_LOOP
      IF(ISEC.LT.NSEC) GO TO 100
C
      CLOSE (UNIT=3, STATUS='DELETE')
      RETURN
      END

C  ****************************************************************************
      SUBROUTINE CHECKUP (XMIN, XMAX, DX)
C
      IMPLICIT NONE
C
      INTEGER*4 N, NX
      REAL*4    XMIN, XMAX, DX, DIFFX
C  ----------------------------------------------------------------------------
C
 100  DIFFX = XMAX-XMIN
      IF(DIFFX.EQ.0.0) THEN                  ! COMPUTE ONLY 1 SECTION
        DX     = .000001
        RETURN
      ENDIF
      IF(DIFFX.LT.0.0) THEN                  ! SWITCH XMIN AND XMAX
        XMIN = XMAX
        XMAX = XMIN-DIFFX
        GO TO 100
      ENDIF
      IF(DIFFX.GT.1.0) THEN                  ! TRAP FOR XMAX-XMIN > 1.0
        XMAX = XMIN+1.0
      ENDIF
      IF(DX.LE.0.0) THEN
        XMAX = XMIN
        GO TO 100
      ENDIF
      IF(DIFFX.EQ.1.0) THEN                  ! SET FOR INTEGRAL SAMPLING
        NX   = NINT(DIFFX/DX)
        DX   = DIFFX/NX
        XMAX = XMIN+DIFFX-DX
      ENDIF
C
      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE MAP_OUT     *****              *
C  *                                                                 *
C  *    WRITE THE 3D MAP TO A DISK FILE (UNIT 3).                    *
C  *                                                                 *
C  *    APRIL 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER             *
C  *    LAST MODIFIED  5-AUG-1998 BY TSB (Store MAP in 3D array and  *
C  *     eliminate need for EMSF3DBT.SCR scratch file).              *
C  *******************************************************************
C
      SUBROUTINE MAP_OUT (MAP, NCOL, NROW, NSEC, A_CELL, B_CELL, 
     1 C_CELL, ALP, BET, GAM, T_MATRIX, MAP_MIN, MAP_MAX, 
     2 MAP_FILENAME, MAP_TITLE)

      IMPLICIT NONE

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T) MAPHEAD

      INTEGER*4 NCOL, NROW, NSEC, I, J, K, T_MATRIX(6)
      REAL*4    MAP(NCOL,NROW,NSEC), TMP, SCALE1, SCALE2
      REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM
      REAL*4    MAP_MIN, MAP_MAX, MAP_SCALEF
      CHARACTER MAP_FILENAME*256
      character(len=1) READWRITE(4), MAP_TITLE(80)
C  ----------------------------------------------------------------------------
C
      WRITE(*,'(2(A,E13.6),/)') ' MIN =', MAP_MIN, ' MAX =', MAP_MAX
C  ********************************
C  *    FINAL OUTPUT OF 3D MAP    *
C  ********************************
      CALL PIF_INIT_HEAD (MAPHEAD)
      DO I =1,80
        MAPHEAD%DESCRIPTIONSTR(I) = MAP_TITLE(I)
      ENDDO
      MAPHEAD%LUNUM = 2
      READWRITE(1)  = 'w'
      CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, READWRITE)
      MAPHEAD%MODE      =   20
      MAPHEAD%NUMCOLS   = NCOL
      MAPHEAD%NUMROWS   = NROW
      MAPHEAD%NUMSECS   = NSEC
      SCALE1            = 1.01*ABS(MAP_MIN/REAL(-32767))
      SCALE2            = 1.01*ABS(MAP_MAX/REAL(32767))
      MAP_SCALEF        = 1.0/MAX(SCALE1,SCALE2)
      MAPHEAD%SCALEF    = MAP_SCALEF
      MAPHEAD%MAP_ABANG = GAM
      MAPHEAD%AOVERB    = 1.0
      MAPHEAD%A_CELL    = A_CELL                        !!! TIMES PIXSIZ
      MAPHEAD%B_CELL    = B_CELL
      MAPHEAD%C_CELL    = C_CELL
      MAPHEAD%DELA      = A_CELL/REAL(NCOL)
      MAPHEAD%DELB      = B_CELL/REAL(NROW)
      MAPHEAD%DELC      = C_CELL/REAL(NSEC)
      DO I = 1,6
        MAPHEAD%T_MATRIX(I)  = T_MATRIX(I)
      ENDDO
      CALL PIF_WRITE_GH (MAPHEAD)
      MAPHEAD%IMGNUM = 1
      CALL PIF_WRITE_DH (MAPHEAD)
C
      PRINT *, '**************************'
      PRINT *, '* Write out final 3D MAP *'
      PRINT *, '**************************'
      call pif_write_map_sections(maphead, map,
     $     maphead%mode, ncol, nrow, nsec, 0, nsec)

      CALL PIF_CLOSE (MAPHEAD%LUNUM)

      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE SFLOAD     *****               *
C  *                                                                    *
C  *    LOAD STRUCTURE FACTOR DATA INTO ARRAYS.                         *
C  *                                                                    *
C  *    OCTOBER 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.  *
C  **********************************************************************
C
      SUBROUTINE SFLOAD (ARRAY, HKL, AP, BP, NSEC, HMAX, KMAX, 
     1 LMAX, NSF, A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX,
     2 SCALE_FAC, NUM_UNIQUE)
C
      IMPLICIT NONE
C
      INTEGER*4 II, JJ, NSEC, HMAX, KMAX, LMAX, NSF, NUM, NUM_UNIQUE
      INTEGER*2 IH, IK, IL, JL, HH, KK, LL, HKL(3,NSF)
      REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
      REAL*4    PI, RADIAN, DEGREE, XSQ, YSQ, ZSQ, DSTAR, MODF, PHASE
      REAL*4    APART, BPART, AP(NSF), BP(NSF), TMP
      COMPLEX   ARRAY(NSEC,HMAX+1,(2*KMAX+1))
C
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      NUM = 0                                          ! SF COUNTER
C
      DO IH = (HMAX+1),2,-1                              ! L < 0 INDICES
        HH  = -(IH - 1)
        DO IK = (2*KMAX+1),1,-1
          KK  = -(IK - KMAX - 1)
          XSQ = (REAL(KK)*A11)**2
          DO 100 IL = -1,-LMAX,-1
            JL = IL + 1 + NSEC
            YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
            ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
            DSTAR = XSQ + YSQ + ZSQ
            IF(DSTAR.GT.DSQMAX .OR. DSTAR.LT.DSQMIN) GO TO 100

            APART      =   REAL(ARRAY(JL,IH,IK))
            BPART      = -AIMAG(ARRAY(JL,IH,IK))
            MODF       = SQRT(APART**2+BPART**2) * SCALE_FAC * 2.0
            PHASE      = 0.0
            IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
            IF(PHASE.LT.0.0)  PHASE = PHASE + 360.0
            NUM        = NUM + 1
            HKL(1,NUM) = HH
            HKL(2,NUM) = KK
            HKL(3,NUM) = -IL
            AP(NUM)    = MODF
            BP(NUM)    = 360.0-PHASE
ccc            AP(NUM)    =   REAL(ARRAY(JL,IH,IK)) * SCALE_FAC * 2.0
ccc            BP(NUM)    = -AIMAG(ARRAY(JL,IH,IK)) * SCALE_FAC * 2.0
  100          ENDDO                                ! LOOP OVER "L" INDEX
          ENDDO                                      ! LOOP OVER "K" INDEX
      ENDDO                                      ! LOOP OVER "H" INDEX
C
      DO IH = 1,HMAX+1                             ! L .GT. 0 INDICES
        HH  = IH - 1
        DO IK = 1,(2*KMAX+1)
          KK  = IK - KMAX - 1
          XSQ = (REAL(KK)*A11)**2
          DO 200 IL = 0,LMAX
            IF(IL.EQ.0) THEN
              IF(HH.EQ.0 .AND. KK.GT.0) GO TO 200
            ENDIF
            YSQ = (REAL(KK)*A21 + REAL(IL)*A22)**2
            ZSQ = (REAL(HH)*A33 + REAL(KK)*A31 + REAL(IL)*A32)**2
            DSTAR = XSQ + YSQ + ZSQ
            IF(DSTAR.GT.DSQMAX .OR. DSTAR.LT.DSQMIN) GO TO 200
            JL         = IL + 1
            APART      =   REAL(ARRAY(JL,IH,IK))
            BPART      = -AIMAG(ARRAY(JL,IH,IK))
            MODF       = SQRT(APART**2+BPART**2) * SCALE_FAC * 2.0
            PHASE      = 0.0
            IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
            IF(PHASE.LT.0.0)  PHASE = PHASE + 360.0
            NUM        = NUM + 1
            HKL(1,NUM) = HH
            HKL(2,NUM) = KK
            HKL(3,NUM) = IL
            AP(NUM)    = MODF
            BP(NUM)    = PHASE
ccc            AP(NUM)    =   REAL(ARRAY(JL,IH,IK)) * SCALE_FAC * 2.0
ccc            BP(NUM)    = -AIMAG(ARRAY(JL,IH,IK)) * SCALE_FAC * 2.0
  200          ENDDO                                ! LOOP OVER "L" INDEX
          ENDDO                                      ! LOOP OVER "K" INDEX
      ENDDO                                      ! LOOP OVER "H" INDEX
C
      WRITE(*,'(A,11X,A,I9)') '+',
     1 'Number of unique structure factors  =', NUM
      NUM_UNIQUE = NUM
C
      RETURN
      END
