      real*4 function phimax (sym, theta)

C     For a given value of theta (the inclination of a view vector from
C     the z-axis) calculate the maximum allowable value of phi that lies
C     within the asymmetric unit (ASU).
C
C     Arguments:
C     sym - symmetry group
C     theta - inclination angle (degrees)
C
C     Return value:
C     phimax - maximum allowable value of phi in the ASU (degrees)
C
C     PURDUE UNIVERSITY DEC 3000, Jan,Jun 1994, T. Baker and D. Belnap
C     Added 222, 522, 622, & 722 symmetries, Dec 1996, Jan 1997, D. Belnap
C     Added 2-20 n-fold symmetries, 30 Jan 1997, D. Belnap
C     24-APR-1998 BY T. BAKER (Renamed and added more SYM.)
C     19-JUN-2006 R. Sinkovits - minor cleanup and commenting

      implicit none

      integer*4, intent(in) :: sym
      real*4, intent(in)    :: theta

      phimax = 180.0            ! for asymmetric objects
      if (sym == 532) then
         call ico_calcphimax (theta, phimax) ! icosahedral symmetry
      else if (sym.ge.2 .and. sym.le.20) then
         phimax = 360.0 / float(sym*2)
      else if (sym == 222) then
         phimax = 360.0/4.0     ! = 90.0 degrees
      else if (sym == 32) then
         phimax = 360.0/6.0     ! = 60.0 degrees
      else if (sym == 422) then
         phimax = 360.0/8.0     ! = 45.0 degrees
      else if (sym == 52) then
         phimax = 360.0/10.0    ! = 36.0 degrees
      else if (sym == 622) then
         phimax = 360.0/12.0    ! = 30.0 degrees
      else if (sym == 72) then
         phimax = 360.0/14.0    ! = 25.7 degrees
      else if (sym == 822) then
         phimax = 360.0/16.0    ! = 22.5 degrees
      else if (sym == 92) then
         phimax = 360.0/18.0    ! = 20.0 degrees
      endif

      return
      end function phimax
